/*! For license information please see 7680.2376c6601e3f3c4fae01.js.LICENSE.txt */
"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[7680,2122,6010,2906],{22122:(r,e,t)=>{function n(){return n=Object.assign||function(r){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(r[n]=t[n])}return r},n.apply(this,arguments)}t.d(e,{Z:()=>n})},19756:(r,e,t)=>{function n(r,e){if(null==r)return{};var t,n,o={},f=Object.keys(r);for(n=0;n<f.length;n++)t=f[n],e.indexOf(t)>=0||(o[t]=r[t]);return o}t.d(e,{Z:()=>n})},28320:(r,e,t)=>{t.d(e,{Z:()=>o});var n=t(71387);function o(r){if("string"!=typeof r)throw new Error((0,n.Z)(7));return r.charAt(0).toUpperCase()+r.slice(1)}},59766:(r,e,t)=>{t.d(e,{P:()=>o,Z:()=>f});var n=t(22122);function o(r){return null!==r&&"object"==typeof r&&r.constructor===Object}function f(r,e,t={clone:!0}){const c=t.clone?(0,n.Z)({},r):r;return o(r)&&o(e)&&Object.keys(e).forEach((n=>{"__proto__"!==n&&(o(e[n])&&n in r&&o(r[n])?c[n]=f(r[n],e[n],t):c[n]=e[n])})),c}},71387:(r,e,t)=>{function n(r){let e="https://mui.com/production-error/?code="+r;for(let r=1;r<arguments.length;r+=1)e+="&args[]="+encodeURIComponent(arguments[r]);return"Minified MUI error #"+r+"; visit "+e+" for the full message."}t.d(e,{Z:()=>n})},47925:(r,e,t)=>{t.d(e,{Z:()=>o});var n=t(22122);function o(r,e){const t=(0,n.Z)({},e);return Object.keys(r).forEach((e=>{void 0===t[e]&&(t[e]=r[e])})),t}},16600:(r,e,t)=>{t.d(e,{Z:()=>o});var n=t(56271);const o="undefined"!=typeof window?n.useLayoutEffect:n.useEffect},86010:(r,e,t)=>{function n(r){var e,t,o="";if("string"==typeof r||"number"==typeof r)o+=r;else if("object"==typeof r)if(Array.isArray(r))for(e=0;e<r.length;e++)r[e]&&(t=n(r[e]))&&(o&&(o+=" "),o+=t);else for(e in r)r[e]&&(o&&(o+=" "),o+=e);return o}function o(){for(var r,e,t=0,o="";t<arguments.length;)(r=arguments[t++])&&(e=n(r))&&(o&&(o+=" "),o+=e);return o}t.d(e,{Z:()=>o})},27418:r=>{var e=Object.getOwnPropertySymbols,t=Object.prototype.hasOwnProperty,n=Object.prototype.propertyIsEnumerable;function o(r){if(null==r)throw new TypeError("Object.assign cannot be called with null or undefined");return Object(r)}r.exports=function(){try{if(!Object.assign)return!1;var r=new String("abc");if(r[5]="de","5"===Object.getOwnPropertyNames(r)[0])return!1;for(var e={},t=0;t<10;t++)e["_"+String.fromCharCode(t)]=t;if("0123456789"!==Object.getOwnPropertyNames(e).map((function(r){return e[r]})).join(""))return!1;var n={};return"abcdefghijklmnopqrst".split("").forEach((function(r){n[r]=r})),"abcdefghijklmnopqrst"===Object.keys(Object.assign({},n)).join("")}catch(r){return!1}}()?Object.assign:function(r,f){for(var c,i,a=o(r),u=1;u<arguments.length;u++){for(var s in c=Object(arguments[u]))t.call(c,s)&&(a[s]=c[s]);if(e){i=e(c);for(var l=0;l<i.length;l++)n.call(c,i[l])&&(a[i[l]]=c[i[l]])}}return a}},75251:(r,e,t)=>{t(27418);var n=t(56271),o=60103;if(e.Fragment=60107,"function"==typeof Symbol&&Symbol.for){var f=Symbol.for;o=f("react.element"),e.Fragment=f("react.fragment")}var c=n.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,i=Object.prototype.hasOwnProperty,a={key:!0,ref:!0,__self:!0,__source:!0};function u(r,e,t){var n,f={},u=null,s=null;for(n in void 0!==t&&(u=""+t),void 0!==e.key&&(u=""+e.key),void 0!==e.ref&&(s=e.ref),e)i.call(e,n)&&!a.hasOwnProperty(n)&&(f[n]=e[n]);if(r&&r.defaultProps)for(n in e=r.defaultProps)void 0===f[n]&&(f[n]=e[n]);return{$$typeof:o,type:r,key:u,ref:s,props:f,_owner:c.current}}e.jsx=u,e.jsxs=u},85893:(r,e,t)=>{r.exports=t(75251)}}]);