"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[6010],{86010:(e,r,n)=>{function t(e){var r,n,f="";if("string"==typeof e||"number"==typeof e)f+=e;else if("object"==typeof e)if(Array.isArray(e))for(r=0;r<e.length;r++)e[r]&&(n=t(e[r]))&&(f&&(f+=" "),f+=n);else for(r in e)e[r]&&(f&&(f+=" "),f+=r);return f}function f(){for(var e,r,n=0,f="";n<arguments.length;)(e=arguments[n++])&&(r=t(e))&&(f&&(f+=" "),f+=r);return f}n.d(r,{Z:()=>f})}}]);