#manual-grade-container {
    margin-left: 30px;
    margin-right: 30px;
}

#info-container {
  margin-bottom: 30px;
}

.manual-grading-list {
  list-style: none;
}

#manual-grade-switch {
  display: inline-flex;
}

#manual-grade-switch > * {
  margin-right: 10px;
}