// Copyright (c) 2022, TU Wien
// All rights reserved.
//
// This source code is licensed under the BSD-style license found in the
// LICENSE file in the root directory of this source tree.

export enum RepoType {
    SOURCE = 'source',
    RELEASE = 'release',
    ASSIGNMENT = 'assignment',
}