"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[2989,4040],{35893:(e,n,s)=>{s.d(n,{Z:()=>t});const t=s(49064).Z},54040:(e,n,s)=>{s.r(n),s.d(n,{capitalize:()=>u.Z,createChainedFunction:()=>a.Z,createSvgIcon:()=>o.Z,debounce:()=>r.Z,deprecatedPropType:()=>i,isMuiElement:()=>c.Z,ownerDocument:()=>l.Z,ownerWindow:()=>d.Z,requirePropFactory:()=>m,setRef:()=>p,unstable_ClassNameGenerator:()=>v,unstable_useEnhancedEffect:()=>Z.Z,unstable_useId:()=>f.Z,unsupportedProp:()=>h,useControlled:()=>b.Z,useEventCallback:()=>g.Z,useForkRef:()=>C.Z,useIsFocusVisible:()=>k.Z});var t=s(31184),u=s(98216),a=s(35893),o=s(82066),r=s(57144);const i=function(e,n){return()=>null};var c=s(77795),l=s(8038),d=s(5340);s(22122);const m=function(e,n){return()=>null},p=s(7960).Z;var Z=s(58974),f=s(27909);const h=function(e,n,s,t,u){return null};var b=s(49299),g=s(2068),C=s(51705),k=s(79674);const v={configure:e=>{console.warn(["MUI: `ClassNameGenerator` import from `@mui/material/utils` is outdated and might cause unexpected issues.","","You should use `import { unstable_ClassNameGenerator } from '@mui/material/className'` instead","","The detail of the issue: https://github.com/mui/material-ui/issues/30011#issuecomment-1024993401","","The updated documentation: https://mui.com/guides/classname-generator/"].join("\n")),t.Z.configure(e)}}}}]);