.nbgrader-TotalPointsInput {
    width: 3em;
    height: 24px;
    text-align: right;
    line-height: 10px;
    margin-left: 0.3em;
    margin-right: 0.3em;
    background: var(--jp-input-background);
    font-size: var(--jp-ui-font-size1);
    font-family: var(--jp-content-font-family);
    color: var(--jp-ui-font-color0);
    flex: 1 1 auto;
    padding: 0 8px 0 8px;
    outline: none;
}

.cellWidget {
    display: flex;
    margin-top: 12px;
    height: auto;
    border-style: solid;
    border-width: var(--jp-border-width) 0 0 0;
    border-color: var(--jp-border-color2);
}

.cellWidget:last-child {
    border-width: 0;
}

.cellWidget>div {
    padding: 0 12px 0 6px;
    border-style: solid;
    border-width: 0 0 0 6px;
    border-color: #0000;
}

.cellWidget.nbgrader-mod-active>div {
    border-color: var(--jp-brand-color1);
}

.cellWidget input {
    background: var(--jp-input-background);
    font-size: var(--jp-ui-font-size1);
    font-family: var(--jp-content-font-family);
    color: var(--jp-ui-font-color0);
    border: var(--jp-border-width) solid var(--jp-border-color1);
    flex: 1 1 auto;
    padding: 0 8px 0 8px;
    outline: none;
}

.cellWidget .jp-select-wrapper {
    display: inline-flex;
    font-size: var(--jp-ui-font-size1);
    margin-bottom: 0;
}

.cellWidget .jp-select-wrapper select.jp-mod-styled {
    font-size: var(--jp-ui-font-size1);
}

.cellWidget .nbgrader-mod-uneditable {
    visibility: hidden;
}

.cellWidget select {
    padding-right: 25px;
}

.cellWidget select:hover {
    cursor: pointer;
}

.nbgrader-TotalPointsInput {
    width: 3em;
    height: 24px;
    text-align: right;
    line-height: 10px;
    margin-left: 0.3em;
    margin-right: 0.3em;
    background: var(--jp-input-background);
    font-size: var(--jp-ui-font-size1);
    font-family: var(--jp-content-font-family);
    color: var(--jp-ui-font-color0);
    border: var(--jp-border-width) solid var(--jp-border-color1);
    flex: 1 1 auto;
    padding: 8px;
    outline: none;
}

.cellWidget {
    display: flex;
    flex-direction: row;
    margin-top: 6px;
    height: auto;
}

.cellWidget:last-child {
    border-width: 0;
}

.cellWidget>div {
    display: flex;
    flex-direction: row;
    padding: 12px 0px 6px 0px;
}

.cellWidget input {
    background: var(--jp-input-background);
    font-size: var(--jp-ui-font-size1);
    font-family: var(--jp-content-font-family);
    color: var(--jp-ui-font-color0);
    border: var(--jp-border-width) solid var(--jp-border-color1);
    flex: 1 1 auto;
    padding: 0 8px 0 8px;
    outline: none;
}

.cellWidget .jp-select-wrapper {
    display: inline-flex;
    font-size: var(--jp-ui-font-size1);
    margin-bottom: 0;
}

.cellWidget .jp-select-wrapper select.jp-mod-styled {
    font-size: var(--jp-ui-font-size1);
}

.cellWidget .nbgrader-mod-uneditable {
    visibility: hidden;
}

.cellWidget select {
    padding-right: 25px;
}

.cellWidget select:hover {
    cursor: pointer;
}

.nbgrader-NotebookPoints,
.cellHeader,
.cellType,
.cellPoints,
.cellComment,
.cellId {
    margin-bottom: 1px;
}

.cellWidget>label {
    font-size: var(--jp-ui-font-size1);
}

.cellHeader {
    display: grid;
    grid-template-columns: auto max-content;
    background-color: #EEE;
    padding: 0 0.5em;
}

.nbgrader-mod-highlight .cellHeader {
    background-color: #337AB7;
}

.cellHeader>.jp-InputPrompt {
    border-width: 0;
    padding-left: 0;
    text-align: left;
    margin: auto 0;
}

.cellHeader>* {
    color: black;
}

.nbgrader-mod-highlight .cellHeader>* {
    color: white;
}

.cellHeader>.nbgrader-LockButton {
    margin: auto 0;
    padding: 0.25em;
    cursor: default;
}

.nbgrader-LockButton {
    opacity: 0;
}

.nbgrader-LockButton.nbgrader-mod-locked {
    opacity: 1;
}

.cellPoints {
    height: 22px;
    text-align: right;
}

.cellId {
    height: 22px;
    text-align: left;
}

.cellComment {
    height: 22px;
}

.cellComment input {
    width: 500px;
    height: 22px;
    text-align: left;
    line-height: 10px;
}

.cellHeader,
.cellType,
.cellPoints,
.cellId,
.cellComment {
    margin-bottom: 12px;
    margin-right: 12px;
}

.cellWidget>label {
    font-size: var(--jp-ui-font-size1);
}

.cellHeader {
    display: grid;
    grid-template-columns: auto max-content;
    background-color: #EEE;
    padding: 0 0.5em;
}

.nbgrader-mod-highlight .cellHeader {
    background-color: #337AB7;
}

.cellHeader>.jp-InputPrompt {
    border-width: 0;
    padding-left: 0;
    text-align: left;
    margin: auto 0;
}

.cellHeader>* {
    color: black;
}

.nbgrader-mod-highlight .cellHeader>* {
    color: white;
}

.cellHeader>.nbgrader-LockButton {
    margin: auto 0;
    padding: 0.25em;
    cursor: default;
}

.nbgrader-LockButton {
    opacity: 0;
}

.nbgrader-LockButton.nbgrader-mod-locked {
    opacity: 1;
}

.cellPoints input {
    width: 3.5em;
    height: 22px;
    text-align: right;
    line-height: 10px;
}

.cellId input {
    height: 22px;
    text-align: left;
}